public inbox for gcc-cvs@sourceware.org
help / color / mirror / Atom feed
* [gcc r9-10178] c++: missing dtor with -fno-elide-constructors [PR100838]
@ 2022-05-13 18:24 Jason Merrill
  0 siblings, 0 replies; only message in thread
From: Jason Merrill @ 2022-05-13 18:24 UTC (permalink / raw)
  To: gcc-cvs

https://gcc.gnu.org/g:200d477d3cb1593dbaa7739c6270e0a7f6c564cf

commit r9-10178-g200d477d3cb1593dbaa7739c6270e0a7f6c564cf
Author: Jason Merrill <jason@redhat.com>
Date:   Mon May 31 12:36:25 2021 -0400

    c++: missing dtor with -fno-elide-constructors [PR100838]
    
    tf_no_cleanup only applies to the outermost TARGET_EXPR, and we already
    clear it for nested calls in build_over_call, but in this case both
    constructor calls came from convert_like, so we need to clear it in the
    recursive call as well.  This revealed that we were adding an extra
    ck_rvalue in direct-initialization cases where it was wrong.
    
            PR c++/100838
            PR c++/105265
    
    gcc/cp/ChangeLog:
    
            * call.c (convert_like_internal): Clear tf_no_cleanup when
            recursing.
            (build_user_type_conversion_1): Only add ck_rvalue if
            LOOKUP_ONLYCONVERTING.
    
    gcc/testsuite/ChangeLog:
    
            * g++.dg/init/no-elide2.C: New test.
            * g++.dg/cpp0x/initlist-new6.C: New test.

Diff:
---
 gcc/cp/call.c                              |  6 +++--
 gcc/testsuite/g++.dg/cpp0x/initlist-new6.C | 39 ++++++++++++++++++++++++++++++
 gcc/testsuite/g++.dg/init/no-elide2.C      | 32 ++++++++++++++++++++++++
 3 files changed, 75 insertions(+), 2 deletions(-)

diff --git a/gcc/testsuite/g++.dg/cpp0x/initlist-new6.C b/gcc/testsuite/g++.dg/cpp0x/initlist-new6.C
new file mode 100644
index 00000000000..0ef27806acf
--- /dev/null
+++ b/gcc/testsuite/g++.dg/cpp0x/initlist-new6.C
@@ -0,0 +1,39 @@
+// PR c++/105265
+// { dg-do run { target c++11 } }
+
+int c;
+
+class Block
+{
+public:
+  Block(int n) : data{new char[n]}, size{n}
+  {
+    ++c;
+  }
+
+  ~Block()
+  {
+    --c;
+    delete[] data;
+  }
+
+private:
+  char* data;
+  int size;
+};
+
+struct Cargo
+{
+  Block const& block;
+};
+
+int main()
+{
+  {
+    Cargo* c = new Cargo{{4000}};
+    delete c;
+  }
+  if (c != 0)
+    __builtin_abort ();
+  return 0;
+}
diff --git a/gcc/testsuite/g++.dg/init/no-elide2.C b/gcc/testsuite/g++.dg/init/no-elide2.C
new file mode 100644
index 00000000000..9a0ba1936ab
--- /dev/null
+++ b/gcc/testsuite/g++.dg/init/no-elide2.C
@@ -0,0 +1,32 @@
+// PR c++/100838
+// { dg-do run }
+// { dg-additional-options -fno-elide-constructors }
+
+extern "C" int puts (const char *);
+
+int c,d;
+class MyString {
+public:
+  MyString(const char* s = "") {
+    puts ("ctor");
+    ++c;
+  }
+  ~MyString() {
+    puts ("dtor");
+    ++d;
+  }
+  MyString(const MyString& s) {
+    puts ("copy ctor");
+    ++c;
+  }
+  MyString& operator=(const MyString& s);
+};
+
+int main() {
+  {
+    MyString s1 = "Hello";
+    puts ("main");
+  }
+  if (c != d)
+    __builtin_abort();
+}


^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2022-05-13 18:24 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2022-05-13 18:24 [gcc r9-10178] c++: missing dtor with -fno-elide-constructors [PR100838] Jason Merrill

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).
diff --git a/gcc/cp/call.c b/gcc/cp/call.c
index 53ca229b909..47db70a8f0b 100644
--- a/gcc/cp/call.c
+++ b/gcc/cp/call.c
@@ -3959,7 +3959,7 @@ build_user_type_conversion_1 (tree totype, tree expr, int flags,
 	{
 	  cand->second_conv = build_identity_conv (totype, NULL_TREE);
 
 	  /* If totype isn't a reference, and LOOKUP_NO_TEMP_BIND isn't
 	     set, then this is copy-initialization.  In that case, "The
 	     result of the call is then used to direct-initialize the
 	     object that is the destination of the copy-initialization."
@@ -3968,6 +3968,8 @@ build_user_type_conversion_1 (tree totype, tree expr, int flags,
 	     We represent this in the conversion sequence with an
 	     rvalue conversion, which means a constructor call.  */
 	  if (!TYPE_REF_P (totype)
+	      && cxx_dialect < cxx17
+	      && (flags & LOOKUP_ONLYCONVERTING)
 	      && !(convflags & LOOKUP_NO_TEMP_BIND))
 	    cand->second_conv
 	      = build_conv (ck_rvalue, totype, cand->second_conv);
@@ -7314,7 +7316,7 @@ convert_like_real (conversion *convs, tree expr, tree fn, int argnum,
   expr = convert_like_real (next_conversion (convs), expr, fn, argnum,
 			    convs->kind == ck_ref_bind
 			    ? issue_conversion_warnings : false, 
-			    c_cast_p, complain);
+			    c_cast_p, complain & ~tf_no_cleanup);
   if (expr == error_mark_node)
     return error_mark_node;
 
@@ -3959,7 +3959,7 @@ build_user_type_conversion_1 (tree totype, tree expr, int flags,
 	  cand->second_conv = build_identity_conv (totype, NULL_TREE);
 
-	  /* If totype isn't a reference, and LOOKUP_NO_TEMP_BIND isn't
+	  /* If totype isn't a reference, and LOOKUP_ONLYCONVERTING is
 	     set, then this is copy-initialization.  In that case, "The
 	     result of the call is then used to direct-initialize the